@@ -1,11 +1,14 @@
-# Copyright (c) 2015-2016, 2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2015-2016, 2018-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2015-2016 Ceridwen <ceridwenv@gmail.com>
 # Copyright (c) 2015 Florian Bruhin <me@the-compiler.org>
 # Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
+# Copyright (c) 2018 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
+# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+# For details: https://github.com/PyCQA/astroid/blob/master/LICENSE
 
 
 """
@@ -19,14 +22,14 @@ leads to an inferred FrozenSet:
 
 import builtins
 
-from astroid import bases
-from astroid import decorators
-from astroid import exceptions
-from astroid import MANAGER
-from astroid import node_classes
-from astroid import scoped_nodes
-from astroid import util
-
+from astroid import bases, decorators, node_classes, scoped_nodes, util
+from astroid.exceptions import (
+    AttributeInferenceError,
+    InferenceError,
+    MroError,
+    SuperError,
+)
+from astroid.manager import AstroidManager
 
 BUILTINS = builtins.__name__
 objectmodel = util.lazy_import("interpreter.objectmodel")
@@ -43,7 +46,7 @@ class FrozenSet(node_classes._BaseContainer):
 
     @decorators.cachedproperty
     def _proxied(self):  # pylint: disable=method-hidden
-        ast_builtins = MANAGER.astroid_cache[BUILTINS]
+        ast_builtins = AstroidManager().builtins_module
         return ast_builtins.getattr("frozenset")[0]
 
 
@@ -63,13 +66,13 @@ class Super(node_classes.NodeNG):
     # pylint: disable=unnecessary-lambda
     special_attributes = util.lazy_descriptor(lambda: objectmodel.SuperModel())
 
-    # pylint: disable=super-init-not-called
     def __init__(self, mro_pointer, mro_type, self_class, scope):
         self.type = mro_type
         self.mro_pointer = mro_pointer
         self._class_based = False
         self._self_class = self_class
         self._scope = scope
+        super().__init__()
 
     def _infer(self, context=None):
         yield self
@@ -77,7 +80,7 @@ class Super(node_classes.NodeNG):
     def super_mro(self):
         """Get the MRO which will be used to lookup attributes in this super."""
         if not isinstance(self.mro_pointer, scoped_nodes.ClassDef):
-            raise exceptions.SuperError(
+            raise SuperError(
                 "The first argument to super must be a subtype of "
                 "type, not {mro_pointer}.",
                 super_=self,
@@ -90,20 +93,18 @@ class Super(node_classes.NodeNG):
         else:
             mro_type = getattr(self.type, "_proxied", None)
             if not isinstance(mro_type, (bases.Instance, scoped_nodes.ClassDef)):
-                raise exceptions.SuperError(
+                raise SuperError(
                     "The second argument to super must be an "
                     "instance or subtype of type, not {type}.",
                     super_=self,
                 )
 
         if not mro_type.newstyle:
-            raise exceptions.SuperError(
-                "Unable to call super on old-style classes.", super_=self
-            )
+            raise SuperError("Unable to call super on old-style classes.", super_=self)
 
         mro = mro_type.mro()
         if self.mro_pointer not in mro:
-            raise exceptions.SuperError(
+            raise SuperError(
                 "The second argument to super must be an "
                 "instance or subtype of type, not {type}.",
                 super_=self,
@@ -114,7 +115,7 @@ class Super(node_classes.NodeNG):
 
     @decorators.cachedproperty
     def _proxied(self):
-        ast_builtins = MANAGER.astroid_cache[BUILTINS]
+        ast_builtins = AstroidManager().builtins_module
         return ast_builtins.getattr("super")[0]
 
     def pytype(self):
@@ -142,8 +143,8 @@ class Super(node_classes.NodeNG):
             mro = self.super_mro()
         # Don't let invalid MROs or invalid super calls
         # leak out as is from this function.
-        except exceptions.SuperError as exc:
-            raise exceptions.AttributeInferenceError(
+        except SuperError as exc:
+            raise AttributeInferenceError(
                 (
                     "Lookup for {name} on {target!r} because super call {super!r} "
                     "is invalid."
@@ -153,8 +154,8 @@ class Super(node_classes.NodeNG):
                 context=context,
                 super_=exc.super_,
             ) from exc
-        except exceptions.MroError as exc:
-            raise exceptions.AttributeInferenceError(
+        except MroError as exc:
+            raise AttributeInferenceError(
                 (
                     "Lookup for {name} on {target!r} failed because {cls!r} has an "
                     "invalid MRO."
@@ -184,19 +185,25 @@ class Super(node_classes.NodeNG):
                     yield inferred
                 elif self._class_based or inferred.type == "staticmethod":
                     yield inferred
+                elif isinstance(inferred, Property):
+                    function = inferred.function
+                    try:
+                        yield from function.infer_call_result(
+                            caller=self, context=context
+                        )
+                    except InferenceError:
+                        yield util.Uninferable
                 elif bases._is_property(inferred):
                     # TODO: support other descriptors as well.
                     try:
                         yield from inferred.infer_call_result(self, context)
-                    except exceptions.InferenceError:
+                    except InferenceError:
                         yield util.Uninferable
                 else:
                     yield bases.BoundMethod(inferred, cls)
 
         if not found:
-            raise exceptions.AttributeInferenceError(
-                target=self, attribute=name, context=context
-            )
+            raise AttributeInferenceError(target=self, attribute=name, context=context)
 
     def getattr(self, name, context=None):
         return list(self.igetattr(name, context=context))
@@ -210,10 +217,13 @@ class ExceptionInstance(bases.Instance):
     the case of .args.
     """
 
-    # pylint: disable=unnecessary-lambda
-    special_attributes = util.lazy_descriptor(
-        lambda: objectmodel.ExceptionInstanceModel()
-    )
+    @decorators.cachedproperty
+    def special_attributes(self):
+        qname = self.qname()
+        instance = objectmodel.BUILTIN_EXCEPTIONS.get(
+            qname, objectmodel.ExceptionInstanceModel
+        )
+        return instance()(self)
 
 
 class DictInstance(bases.Instance):
@@ -245,6 +255,58 @@ class DictValues(bases.Proxy):
     __repr__ = node_classes.NodeNG.__repr__
 
 
+class PartialFunction(scoped_nodes.FunctionDef):
+    """A class representing partial function obtained via functools.partial"""
+
+    def __init__(
+        self, call, name=None, doc=None, lineno=None, col_offset=None, parent=None
+    ):
+        super().__init__(name, doc, lineno, col_offset, parent)
+        self.filled_positionals = len(call.positional_arguments[1:])
+        self.filled_args = call.positional_arguments[1:]
+        self.filled_keywords = call.keyword_arguments
+
+    def infer_call_result(self, caller=None, context=None):
+        if context:
+            current_passed_keywords = {
+                keyword for (keyword, _) in context.callcontext.keywords
+            }
+            for keyword, value in self.filled_keywords.items():
+                if keyword not in current_passed_keywords:
+                    context.callcontext.keywords.append((keyword, value))
+
+            call_context_args = context.callcontext.args or []
+            context.callcontext.args = self.filled_args + call_context_args
+
+        return super().infer_call_result(caller=caller, context=context)
+
+    def qname(self):
+        return self.__class__.__name__
+
+
 # TODO: Hack to solve the circular import problem between node_classes and objects
 # This is not needed in 2.0, which has a cleaner design overall
 node_classes.Dict.__bases__ = (node_classes.NodeNG, DictInstance)
+
+
+class Property(scoped_nodes.FunctionDef):
+    """Class representing a Python property"""
+
+    def __init__(
+        self, function, name=None, doc=None, lineno=None, col_offset=None, parent=None
+    ):
+        self.function = function
+        super().__init__(name, doc, lineno, col_offset, parent)
+
+    # pylint: disable=unnecessary-lambda
+    special_attributes = util.lazy_descriptor(lambda: objectmodel.PropertyModel())
+    type = "property"
+
+    def pytype(self):
+        return "%s.property" % BUILTINS
+
+    def infer_call_result(self, caller=None, context=None):
+        raise InferenceError("Properties are not callable")
+
+    def infer(self, context=None, **kwargs):
+        return iter((self,))
