@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright (c) 2006-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
 # Copyright (c) 2012 FELD Boris <lothiraldan@gmail.com>
 # Copyright (c) 2014-2020 Claudiu Popa <pcmanticore@gmail.com>
@@ -11,12 +10,15 @@
 # Copyright (c) 2018 Ville Skyttä <ville.skytta@iki.fi>
 # Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
 # Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
-# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2020-2021 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2020 Becker Awqatty <bawqatty@mide.com>
 # Copyright (c) 2020 Robin Jarry <robin.jarry@6wind.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
+# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Andrew Haigh <hello@nelf.in>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+# For details: https://github.com/PyCQA/astroid/blob/master/LICENSE
 
 """this module contains a set of functions to create astroid trees from scratch
 (build_* functions) or from living object (object_build_* functions)
@@ -28,16 +30,12 @@ import os
 import sys
 import types
 import warnings
+from typing import List, Optional
 
-from astroid import bases
-from astroid import manager
-from astroid import node_classes
-from astroid import nodes
+from astroid import bases, node_classes, nodes
+from astroid.manager import AstroidManager
 
-
-MANAGER = manager.AstroidManager()
 # the keys of CONST_CLS eg python builtin types
-
 _CONSTANTS = tuple(node_classes.CONST_CLS)
 _BUILTINS = vars(builtins)
 TYPE_NONE = type(None)
@@ -126,29 +124,33 @@ def build_class(name, basenames=(), doc=None):
     return node
 
 
-def build_function(name, args=None, posonlyargs=None, defaults=None, doc=None):
+def build_function(
+    name,
+    args: Optional[List[str]] = None,
+    posonlyargs: Optional[List[str]] = None,
+    defaults=None,
+    doc=None,
+    kwonlyargs: Optional[List[str]] = None,
+) -> nodes.FunctionDef:
     """create and initialize an astroid FunctionDef node"""
-    args, defaults, posonlyargs = args or [], defaults or [], posonlyargs or []
     # first argument is now a list of decorators
     func = nodes.FunctionDef(name, doc)
-    func.args = argsnode = nodes.Arguments()
-    argsnode.args = []
-    argsnode.posonlyargs = []
-    for arg in args:
-        argsnode.args.append(nodes.Name())
-        argsnode.args[-1].name = arg
-        argsnode.args[-1].parent = argsnode
-    for arg in posonlyargs:
-        argsnode.posonlyargs.append(nodes.Name())
-        argsnode.posonlyargs[-1].name = arg
-        argsnode.posonlyargs[-1].parent = argsnode
-    argsnode.defaults = []
-    for default in defaults:
+    func.args = argsnode = nodes.Arguments(parent=func)
+    argsnode.postinit(
+        args=[nodes.AssignName(name=arg, parent=argsnode) for arg in args or ()],
+        defaults=[],
+        kwonlyargs=[
+            nodes.AssignName(name=arg, parent=argsnode) for arg in kwonlyargs or ()
+        ],
+        kw_defaults=[],
+        annotations=[],
+        posonlyargs=[
+            nodes.AssignName(name=arg, parent=argsnode) for arg in posonlyargs or ()
+        ],
+    )
+    for default in defaults or ():
         argsnode.defaults.append(nodes.const_factory(default))
         argsnode.defaults[-1].parent = argsnode
-    argsnode.kwarg = None
-    argsnode.vararg = None
-    argsnode.parent = func
     if args:
         register_arguments(func)
     return func
@@ -172,7 +174,7 @@ def register_arguments(func, args=None):
         if func.args.kwarg:
             func.set_local(func.args.kwarg, func.args)
     for arg in args:
-        if isinstance(arg, nodes.Name):
+        if isinstance(arg, nodes.AssignName):
             func.set_local(arg.name, arg)
         else:
             register_arguments(func, arg.elts)
@@ -190,6 +192,7 @@ def object_build_function(node, member, localname):
     args = []
     defaults = []
     posonlyargs = []
+    kwonlyargs = []
     for param_name, param in signature.parameters.items():
         if param.kind == inspect.Parameter.POSITIONAL_ONLY:
             posonlyargs.append(param_name)
@@ -199,6 +202,8 @@ def object_build_function(node, member, localname):
             args.append(param_name)
         elif param.kind == inspect.Parameter.VAR_KEYWORD:
             args.append(param_name)
+        elif param.kind == inspect.Parameter.KEYWORD_ONLY:
+            kwonlyargs.append(param_name)
         if param.default is not inspect._empty:
             defaults.append(param.default)
     func = build_function(
@@ -293,7 +298,8 @@ class InspectBuilder:
     FunctionDef and ClassDef nodes and some others as guessed.
     """
 
-    def __init__(self):
+    def __init__(self, manager_instance=None):
+        self._manager = manager_instance or AstroidManager()
         self._done = {}
         self._module = None
 
@@ -312,7 +318,7 @@ class InspectBuilder:
             node = build_module(modname)
         node.file = node.path = os.path.abspath(path) if path else path
         node.name = modname
-        MANAGER.cache_module(node)
+        self._manager.cache_module(node)
         node.package = hasattr(module, "__path__")
         self._done = {}
         self.object_build(node, module)
@@ -320,7 +326,7 @@ class InspectBuilder:
 
     def object_build(self, node, obj):
         """recursive method which create a partial ast from real objects
-         (only function, class, and method are handled)
+        (only function, class, and method are handled)
         """
         if obj in self._done:
             return self._done[obj]
@@ -417,8 +423,9 @@ class InspectBuilder:
 
 _CONST_PROXY = {}
 
-# TODO : find a nicer way to handle this situation;
+
 def _set_proxied(const):
+    # TODO : find a nicer way to handle this situation;
     return _CONST_PROXY[const.value.__class__]
 
 
@@ -429,7 +436,6 @@ def _astroid_bootstrapping():
     builder = InspectBuilder()
     astroid_builtin = builder.inspect_build(builtins)
 
-    # pylint: disable=redefined-outer-name
     for cls, node_cls in node_classes.CONST_CLS.items():
         if cls is TYPE_NONE:
             proxy = build_class("NoneType")
@@ -458,7 +464,6 @@ def _astroid_bootstrapping():
     builder.object_build(bases.Generator._proxied, types.GeneratorType)
 
     if hasattr(types, "AsyncGeneratorType"):
-        # pylint: disable=no-member; AsyncGeneratorType
         _AsyncGeneratorType = nodes.ClassDef(
             types.AsyncGeneratorType.__name__, types.AsyncGeneratorType.__doc__
         )
